﻿//Kod kreskowy Itf-14
//(c) 2019 by Tomasz Lubinski
//www.algorytm.org

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Imaging;

namespace Barcodes.Itf14
{
    public partial class Form1 : Form
    {
        private readonly char[] code25signs = 
        {
            '0', '1', '2', '3', '4', '5', '6', '7', '8', '9'
        };
 
        private readonly char[,] code25bars = 
        {
            {'N', 'N', 'W', 'W', 'N'},
            {'W', 'N', 'N', 'N', 'W'},
            {'N', 'W', 'N', 'N', 'W'},
            {'W', 'W', 'N', 'N', 'N'},
            {'N', 'N', 'W', 'N', 'W'},
            {'W', 'N', 'W', 'N', 'N'},
            {'N', 'W', 'W', 'N', 'N'},
            {'N', 'N', 'N', 'W', 'W'},
            {'W', 'N', 'N', 'W', 'N'},
            {'N', 'W', 'N', 'W', 'N'}
        };

        private readonly char[] start = { 'N', 'N', 'N', 'N' };
        private readonly char[] stop = { 'W', 'N', 'N' };

        /// <summary>
        /// Zainicjuj
        /// </summary>
        public Form1()
        {
            InitializeComponent();
            pictureBox.Image = new Bitmap(pictureBox.Width, pictureBox.Height, PixelFormat.Format24bppRgb);
            System.Drawing.SolidBrush whiteBrush = new System.Drawing.SolidBrush(System.Drawing.Color.White);
            System.Drawing.Graphics graphics = Graphics.FromImage(pictureBox.Image);
            graphics.FillRectangle(whiteBrush, new Rectangle(0, 0, pictureBox.Width, pictureBox.Height));
            pictureBox.Refresh();
            whiteBrush.Dispose();
            graphics.Dispose();
        }

        /// <summary>
        /// Sprawdza czy podany txt mozna zakodowac ITF-14
        /// </summary>
        /// <param name="txt">tekst do zakodowania</param>
        /// <returns>true jezeli mozna zakodowac, false w przeciwnym razie</returns>
        public bool check(string txt)
        {
            //Sprawdz dlugosc
            if (txt.Length != 14)
            {
                return false;
            }
            //Sprawdz czy podano tylko kodowalne znaki
            for (int i = 0; i < txt.Length; i++)
            {
                if (checkLetter(txt[i]) == false)
                {
                    return false;
                }
            }
            //Sprawdz cyfre kontrolna
            if (checkDigit(txt) != txt[13])
            {
                return false;
            }

            return true;
        }

        /// <summary>
        /// Oblicza cyfre kontrolna
        /// </summary>
        /// <param name="txt">Ciag dla ktorego obliczna jest cyfra kontrolna</param>
        /// <returns>Cyfra kontrolna dla podanego ciagu</returns>
        private char checkDigit(string txt)
        {
            int sum = 3 * getLetterValue(txt[0]) +
            1 * getLetterValue(txt[1]) +
            3 * getLetterValue(txt[2]) +
            1 * getLetterValue(txt[3]) +
            3 * getLetterValue(txt[4]) +
            1 * getLetterValue(txt[5]) +
            3 * getLetterValue(txt[6]) +
            1 * getLetterValue(txt[7]) +
            3 * getLetterValue(txt[8]) +
            1 * getLetterValue(txt[9]) +
            3 * getLetterValue(txt[10]) +
            1 * getLetterValue(txt[11]) +
            3 * getLetterValue(txt[12]);

            sum %= 10;
            sum = 10 - sum;
            sum %= 10;

            return code25signs[sum];
        } 

        /// <summary>
        /// Sprawdza czy podany znak mozna zakodowac ITF-14
        /// </summary>
        /// <param name="ch">znak do zakodowania</param>
        /// <returns>true jezeli mozna zakodowac, false w przeciwnym razie</returns>
        private bool checkLetter(char ch)
        {
            for (int i = 0; i < code25signs.Length; i++)
            {
                if (code25signs[i] == ch)
                    return true;
            }
            return false;
        }

        /// <summary>
        /// Zwraca wartosc danego znaku
        /// </summary>
        /// <param name="ch">znak</param>
        /// <returns>wartosc danego znaku</returns>
        private int getLetterValue(char ch)
        {
            for (int i = 0; i < code25signs.Length; i++)
                if (code25signs[i] == ch)
                    return i;

            return -1;
        }

        /// <summary>
        /// Generuj ITF-14
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Generate_Click(object sender, EventArgs e)
        {
            //Zainicjuj narzedzia do rysowania
            System.Drawing.SolidBrush whiteBrush = new System.Drawing.SolidBrush(System.Drawing.Color.White);
            System.Drawing.SolidBrush blackBrush = new System.Drawing.SolidBrush(System.Drawing.Color.Black);
            System.Drawing.Graphics graphics = Graphics.FromImage(pictureBox.Image);
            Font drawFont = new Font("Arial", 10);

            //Wyczysc obszar rysowania
            graphics.FillRectangle(whiteBrush, new Rectangle(0, 0, pictureBox.Width, pictureBox.Height));

            //Sprawdz czy podany ciag mozna zakodowac
            string code = Code.Text;
            if (check(code) == false)
            {
                MessageBox.Show("Podany kod jest nieprawidłowy!", "Bład", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            //Wygeneruj i narysuj kod
            char[] bars = generateBars(code);
            int pos = 40;
            int width = 0;
            for (int i = 0; i < bars.Length; i++)
            {
                //Sprawdz czy teraz bedzie pasek szeroki czy waski
                if (bars[i] == 'N')
                {
                    width = 2;
                }
                else
                {
                    width = 4;
                }
                //Sprawdz czy pasek bialy czy czarny (rysujemy tylko czarne, bo tlo jest biale)
                if ((i % 2) == 0)
                {
                    graphics.FillRectangle(blackBrush, pos, 10, width, 90);
                }
                pos += width;
            }

            //Wypisz kodowany ciag pod kodem kreskowym
            for (int i = 0; i < code.Length; i++)
            {
                graphics.DrawString(code.Substring(i, 1), drawFont, blackBrush, i * 14 + 50, 102);
            }


            //Odswiez
            pictureBox.Refresh();

            //Zwolinij zasoby
            whiteBrush.Dispose();
            blackBrush.Dispose();
            graphics.Dispose();
        }

        /// <summary>
        /// Generuje kod kreskowy
        /// </summary>
        /// <param name="code">string do zakodowania kodem kreskowym</param>
        /// <returns>kod kreskowy - W pasek szeroki, N - pasek waski</returns>
        private char[] generateBars(string code)
        {
            char[] bars = new char[5 * code.Length + 7];

            //Dodaj znak start/stop
            for (int j = 0; j < 4; j++)
            {
                bars[j] = start[j];
            }

            //Dodaj kolejne znaki (parami)
            for (int i = 0; i < code.Length; i+=2)
            {
                int letterValueBars = getLetterValue(code[i]);
                int letterValueSpaces = getLetterValue(code[i + 1]);
                for (int j = 0; j < 5; j++)
                {
                    bars[i * 5 + 4 + j * 2] = code25bars[letterValueBars, j];
                    bars[i * 5 + 5 + j * 2] = code25bars[letterValueSpaces, j];
                }
            }

            //Dodaj znak stop
            for (int j = 0; j < 3; j++)
            {
                bars[5 * code.Length + 4 + j] = stop[j];
            }
            
            return bars;
        }
    }
}
